#include "Framework/BaseObject.h"
#include "Engine/Engine.h"
#include "Log.h"

BaseObject::BaseObject(FluxEngine* Engine)
{
    _CachedEngine = Engine;
}

BaseObject::~BaseObject()
{
    LOG_INFO("Destroying object: {0}", GetName());
}

void BaseObject::Init()
{
    if (bIsInitialized || bUnreachable)
    {
        return;
    }

    _Name = std::string(typeid(*this).name()) + "_" + std::to_string(reinterpret_cast<std::uintptr_t>(this));
    bIsInitialized = true;
}

void BaseObject::Destroy()
{
    if (bUnreachable)
    {
        return;
    }

    bUnreachable = true;
}

bool BaseObject::IsValid()
{
    return bIsInitialized;
}

bool BaseObject::IsUnreachable(BaseObject* Object)
{
    return Object && Object->IsValid();
}

std::string BaseObject::GetName() const
{
    return _Name;
}

FluxEngine* BaseObject::GetEngine() const
{
    if (_CachedEngine == nullptr)
    {
       _CachedEngine = FluxEngine::GetInstance();
    }
    
    return _CachedEngine;
}